import { Component, OnDestroy, OnInit } from '@angular/core';
import { AbstractControl, FormArray, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

enum TableType {
  /** 普通表 */
  ordinary = 'ordinary',
  /** 分区表 */
  partition = 'partition',
  /** 分表 */
  sub = 'sub'
}

interface FormArrayItem {
  paramKey: string;
  paramValue: Validators;
}

@Component({
  selector: '<%= selector %>',
  <% if(inlineTemplate) { %>template: `
    <form
      nz-form
      [nzLayout]="validateForm.get('formLayout')?.value"
      [formGroup]="validateForm"
      (ngSubmit)="submitForm()"
    >
      <nz-form-item>
        <nz-form-label [nzSpan]="isHorizontal ? 4 : null">Form Layout</nz-form-label>
        <nz-form-control [nzSpan]="isHorizontal ? 14 : null">
          <nz-radio-group formControlName="formLayout">
            <label nz-radio-button [nzValue]="'horizontal'">Horizontal</label>
            <label nz-radio-button [nzValue]="'vertical'">Vertical</label>
            <label nz-radio-button [nzValue]="'inline'">Inline</label>
          </nz-radio-group>
        </nz-form-control>
      </nz-form-item>
      <nz-form-item>
        <nz-form-label [nzSpan]="isHorizontal ? 4 : null">Field A</nz-form-label>
        <nz-form-control [nzSpan]="isHorizontal ? 14 : null">
          <input nz-input formControlName="fieldA" placeholder="input placeholder" />
          <nz-form-explain *ngIf="validateForm.get('fieldA')?.dirty && validateForm.get('fieldA')?.errors"
            >Please input your username!</nz-form-explain
          >
        </nz-form-control>
      </nz-form-item>
      <nz-form-item>
        <nz-form-label [nzSpan]="isHorizontal ? 4 : null">Field B</nz-form-label>
        <nz-form-control [nzSpan]="isHorizontal ? 14 : null">
          <input nz-input formControlName="filedB" placeholder="input placeholder" />
          <nz-form-explain *ngIf="validateForm.get('filedB')?.dirty && validateForm.get('filedB')?.errors"
            >Please input your Password!</nz-form-explain
          >
        </nz-form-control>
      </nz-form-item>

      <ng-container formArrayName="chaildrenFrom">
        <ng-container *ngFor="let item of chaildrenFrom.controls; let i = index">
          <ng-container [formGroupName]="i">
            <nz-form-item *ngIf="searchBoolen(i, 'table_type')">
              <nz-form-label [nzSpan]="isHorizontal ? 4 : null">Select</nz-form-label>
              <nz-form-control [nzSpan]="isHorizontal ? 14 : null">
                <nz-radio-group
                  formControlName="paramValue"
                  [nzButtonStyle]="'solid'"
                  (ngModelChange)="radiogroupTest($event)"
                >
                  <label nz-radio-button nzValue="ordinary">普通表</label>
                  <label nz-radio-button nzValue="partition">分区表</label>
                  <label nz-radio-button nzValue="sub">分表</label>
                </nz-radio-group>
              </nz-form-control>
            </nz-form-item>

            <nz-form-item *ngIf="searchBoolen(i, 'slave_partition_field')">
              <nz-form-label [nzSpan]="isHorizontal ? 4 : null">Input</nz-form-label>
              <nz-form-control [nzSpan]="isHorizontal ? 14 : null">
                <input nz-input placeholder="Basic usage" formControlName="paramValue" />
              </nz-form-control>
            </nz-form-item>

            <nz-form-item *ngIf="searchBoolen(i, 'submeter_suffix')">
              <nz-form-label [nzSpan]="isHorizontal ? 4 : null">Change</nz-form-label>
              <nz-form-control [nzSpan]="isHorizontal ? 14 : null">
                <nz-select style="width: 120px;" nzAllowClear nzPlaceHolder="Choose" formControlName="paramValue">
                  <nz-option nzValue="jack" nzLabel="Jack"></nz-option>
                  <nz-option nzValue="lucy" nzLabel="Lucy"></nz-option>
                  <nz-option nzValue="disabled" nzLabel="Disabled" nzDisabled></nz-option>
                </nz-select>
              </nz-form-control>
            </nz-form-item>
          </ng-container>
        </ng-container>
      </ng-container>

      <nz-form-item>
        <nz-form-control [nzSpan]="isHorizontal ? 14 : null" [nzOffset]="isHorizontal ? 4 : null">
          <button nz-button nzType="primary">Submit</button>
        </nz-form-control>
      </nz-form-item>
    </form>
  `<% } else { %>templateUrl: './<%= dasherize(name) %>.component.html'<% } %>,
  <% if(inlineStyle) { %>styles: [`
      [nz-form]:not(.ant-form-inline):not(.ant-form-vertical) {
        max-width: 600px;
      }
    `]<% } else { %>styleUrls: ['./<%= dasherize(name) %>.component.<%= style %>']<% } %>
})
export class <%= classify(name) %>Component implements OnInit, OnDestroy {
  validateForm: FormGroup;
  componetDestroyed: Subject<TableType> = new Subject<TableType>();

  get chaildrenFrom(): FormArray {
    return this.validateForm.get('chaildrenFrom') as FormArray;
  }

  submitForm(): void {
    for (const i in this.validateForm.controls) {
      this.validateForm.controls[i].markAsDirty();
      this.validateForm.controls[i].updateValueAndValidity();
    }
  }

  get isHorizontal(): boolean {
    return this.validateForm.controls.formLayout && this.validateForm.controls.formLayout.value === 'horizontal';
  }

  constructor(private fb: FormBuilder) {}

  ngOnInit(): void {
    this.validateForm = this.fb.group({
      formLayout: ['horizontal'],
      fieldA: [null, [Validators.required]],
      filedB: [null, [Validators.required]],
      chaildrenFrom: this.createArrayForm()
    });
    this.addListener();
  }

  searchBoolen(i: number, key: string): boolean {
    const childForm = this.chaildrenFrom;
    if (childForm) {
      const controls = childForm.controls;
      if (controls) {
        const controlitem = controls[i];
        if (controlitem) {
          const valueObj = controlitem.get('paramKey');
          if (valueObj) {
            return valueObj.value === key;
          }
        }
      }
    }
    return false;
  }

  addListener(): void {
    const index = this.getKeyForIndex('table_type');
    if (index !== -1) {
      const childForm = this.chaildrenFrom;
      if (childForm) {
        const controls = childForm.controls;
        if (controls) {
          const controlitem = controls[index];
          if (controlitem) {
            const key = controlitem.get('paramValue');
            if (key) {
              key.valueChanges.pipe(takeUntil(this.componetDestroyed)).subscribe((value: TableType) => {
                this.removeControl();
                this.addControl(value);
              });
            }
          }
        }
      }
    }
  }

  getKeyForIndex(key: string): number {
    return this.chaildrenFrom.value.findIndex((item: FormArrayItem) => item.paramKey === key);
  }

  removeControl(): void {
    const arr = ['slave_partition_field', 'submeter_suffix'];
    arr.forEach((key: string) => {
      const data_cycle_index = this.getKeyForIndex(key);
      if (data_cycle_index !== -1) {
        this.chaildrenFrom.removeAt(data_cycle_index);
      }
    });
  }

  addControl(value: TableType): void {
    switch (value) {
      case TableType.ordinary:
        this.chaildrenFrom.push(this.addGroup('slave_partition_field', null));
        break;
      case TableType.partition:
        this.chaildrenFrom.push(this.addGroup('submeter_suffix', null, [Validators.required]));
        break;
      case TableType.sub:
        this.chaildrenFrom.push(this.addGroup('submeter_suffix', null, [Validators.required]));
    }
  }

  addGroup(key: string, value: string | null, validators?: Validators): AbstractControl {
    return this.fb.group({
      paramKey: key,
      paramValue: validators ? [{ value: value, disabled: false }, validators] : [{ value: value, disabled: false }]
    });
  }

  createArrayForm(): FormArray {
    return this.fb.array([
      this.fb.group({
        paramKey: 'table_type',
        paramValue: [{ value: 'partition', disabled: false }, Validators.required]
      })
    ]);
  }

  radiogroupTest(e: string): void {
    console.log(e);
    console.log(this.chaildrenFrom);
  }

  ngOnDestroy(): void {
    this.componetDestroyed.next();
    this.componetDestroyed.unsubscribe();
  }
}
